@@ -1,9 +1,12 @@
+# ------------------------------------------------------------------------
+# coding=utf-8
+# ------------------------------------------------------------------------
+
 import re
 import copy
 
 from django import forms, template
 from django.contrib import admin
-from django.core.exceptions import ImproperlyConfigured
 from django.db.models import loading
 from django.forms.models import modelform_factory
 from django.http import Http404
@@ -14,53 +17,97 @@ from django.utils.translation import ugettext as _
 from django.contrib.admin.options import InlineModelAdmin
 
 from feincms import settings, ensure_completely_loaded
+from feincms.signals import itemeditor_post_save_related
 
+# ------------------------------------------------------------------------
 FRONTEND_EDITING_MATCHER = re.compile(r'(\d+)\|(\w+)\|(\d+)')
 FEINCMS_CONTENT_FIELDSET_NAME = 'FEINCMS_CONTENT'
 FEINCMS_CONTENT_FIELDSET = (FEINCMS_CONTENT_FIELDSET_NAME, {'fields': ()})
 
-
+# ------------------------------------------------------------------------
 class ItemEditorForm(forms.ModelForm):
+    """
+    The item editor form contains hidden region and ordering fields and should
+    be used for all content type inlines.
+    """
+
     region = forms.CharField(widget=forms.HiddenInput())
     ordering = forms.IntegerField(widget=forms.HiddenInput())
 
-
+# ------------------------------------------------------------------------
 class FeinCMSInline(InlineModelAdmin):
+    """
+    Custom ``InlineModelAdmin`` subclass used for content types.
+    """
+
+    form = ItemEditorForm
     extra = 0
     fk_name = 'parent'
     template = 'admin/feincms/content_inline.html'
 
-    def __init__(self, *args, **kwargs):
-        super(FeinCMSInline, self).__init__(*args, **kwargs)
-        self.verbose_name_plural = \
-            u'Feincms_Inline: %s' % (self.verbose_name_plural,) # earmark
-
-
-def get_feincms_inlines(model):
-    """ Generate genuine django inlines for registered content types. """
-    inlines = []
-    for content_type in model._feincms_content_types:
-        name = '%sFeinCMSInline' % content_type.__name__
-        attrs = {
-            '__module__': model.__module__,
-            'model': content_type,
-            'form': getattr(content_type, 'feincms_item_editor_form',
-                            ItemEditorForm),
-            }
-        inlines.append(type(name, (FeinCMSInline,), attrs))
-    return inlines
-
-
+# ------------------------------------------------------------------------
 class ItemEditor(admin.ModelAdmin):
+    """
+    The ``ItemEditor`` is a drop-in replacement for ``ModelAdmin`` with the
+    speciality of knowing how to work with :class:`feincms.models.Base`
+    subclasses and associated content types.
+
+    It does not have any public API except from everything inherited from'
+    the standard ``ModelAdmin`` class.
+    """
+
     def __init__(self, model, admin_site):
         ensure_completely_loaded()
 
         super(ItemEditor, self).__init__(model, admin_site)
 
-        # Add inline instances for FeinCMS content inlines
-        for inline_class in get_feincms_inlines(model):
+        if hasattr(self, 'inline_instances'):
+            # Add inline instances for FeinCMS content inlines
+            # This works in Django 1.3 and lower
+            # In Django 1.4 inline instances are generated using overridden get_inline_instances()
+            self.append_feincms_inlines(self.inline_instances)
+
+    def get_inline_instances(self, request):
+        inline_instances = super(ItemEditor, self).get_inline_instances(request)
+        self.append_feincms_inlines(inline_instances)
+
+        return inline_instances
+
+    def append_feincms_inlines(self, inline_instances):
+        """ Append generated FeinCMS content inlines to native django inlines. """
+        for inline_class in self.get_feincms_inlines(self.model):
             inline_instance = inline_class(self.model, self.admin_site)
-            self.inline_instances.append(inline_instance)
+            inline_instances.append(inline_instance)
+
+    def get_feincms_inlines(self, model):
+        """ Generate genuine django inlines for registered content types. """
+        model._needs_content_types()
+
+        inlines = []
+        for content_type in model._feincms_content_types:
+            attrs = {
+                '__module__': model.__module__,
+                'model': content_type,
+                }
+
+            if hasattr(content_type, 'feincms_item_editor_inline'):
+                inline = content_type.feincms_item_editor_inline
+                attrs['form'] = inline.form
+
+                if hasattr(content_type, 'feincms_item_editor_form'):
+                    import warnings
+                    warnings.warn(
+                        'feincms_item_editor_form on %s is ignored because feincms_item_editor_inline is set too' % content_type,
+                        RuntimeWarning)
+
+            else:
+                inline = FeinCMSInline
+                attrs['form'] = getattr(content_type,
+                    'feincms_item_editor_form', inline.form)
+
+            name = '%sFeinCMSInline' % content_type.__name__
+            inlines.append(type(name, (inline,), attrs))
+        return inlines
 
     def _frontend_editing_view(self, request, cms_id, content_type, content_id):
         """
@@ -104,28 +151,24 @@ class ItemEditor(admin.ModelAdmin):
                 return render_to_response('admin/feincms/fe_editor_done.html', {
                     'content': obj.render(request=request),
                     'identifier': obj.fe_identifier(),
-                    'FEINCMS_ADMIN_MEDIA': settings.FEINCMS_ADMIN_MEDIA,
-                    'FEINCMS_ADMIN_MEDIA_HOTLINKING': \
-                        settings.FEINCMS_ADMIN_MEDIA_HOTLINKING,
                     'FEINCMS_JQUERY_NO_CONFLICT': \
                         settings.FEINCMS_JQUERY_NO_CONFLICT,
                     })
         else:
             form = ModelForm(instance=obj, prefix=content_type)
 
-        return render_to_response('admin/feincms/fe_editor.html', {
+        context = self.get_extra_context(request)
+        context.update({
             'frontend_editing': True,
             'title': _('Change %s') % force_unicode(model_cls._meta.verbose_name),
             'object': obj,
             'form': form,
             'is_popup': True,
             'media': self.media,
-            'FEINCMS_ADMIN_MEDIA': settings.FEINCMS_ADMIN_MEDIA,
-            'FEINCMS_ADMIN_MEDIA_HOTLINKING': \
-                settings.FEINCMS_ADMIN_MEDIA_HOTLINKING,
-            'FEINCMS_JQUERY_NO_CONFLICT': settings.FEINCMS_JQUERY_NO_CONFLICT,
-            }, context_instance=template.RequestContext(request,
-                processors=self.model.feincms_item_editor_context_processors))
+            })
+
+        return render_to_response('admin/feincms/fe_editor.html', context,
+            context_instance=template.RequestContext(request))
 
     def get_content_type_map(self):
         """ Prepare mapping of content types to their prettified names. """
@@ -144,11 +187,10 @@ class ItemEditor(admin.ModelAdmin):
                 getattr(self.model, '_feincms_templates', ()),
             'has_parent_attribute': hasattr(self.model, 'parent'),
             'content_types': self.get_content_type_map(),
-            'FEINCMS_ADMIN_MEDIA': settings.FEINCMS_ADMIN_MEDIA,
-            'FEINCMS_ADMIN_MEDIA_HOTLINKING':
-                settings.FEINCMS_ADMIN_MEDIA_HOTLINKING,
             'FEINCMS_JQUERY_NO_CONFLICT': settings.FEINCMS_JQUERY_NO_CONFLICT,
             'FEINCMS_CONTENT_FIELDSET_NAME': FEINCMS_CONTENT_FIELDSET_NAME,
+
+            'FEINCMS_FRONTEND_EDITING': settings.FEINCMS_FRONTEND_EDITING,
             }
 
         for processor in self.model.feincms_item_editor_context_processors:
@@ -175,8 +217,6 @@ class ItemEditor(admin.ModelAdmin):
         return super(ItemEditor, self).add_view(request, form_url, context)
 
     def change_view(self, request, object_id, extra_context=None):
-        self.model._needs_content_types()
-
         # Recognize frontend editing requests
         # This is done here so that the developer does not need to add
         # additional entries to # urls.py or something...
@@ -190,6 +230,21 @@ class ItemEditor(admin.ModelAdmin):
         context.update(extra_context or {})
         return super(ItemEditor, self).change_view(request, object_id, context)
 
+    # The next two add support for sending a "saving done" signal as soon
+    # as all relevant data have been saved (especially all foreign key relations)
+    # This can be used to keep functionality dependend on item content happy.
+    # NOTE: These two can (and probably should) be replaced by overriding
+    # `save_related` as soon as we don't depend on Django<1.4 any more.
+    def response_add(self, request, obj, *args, **kwargs):
+        r = super(ItemEditor, self).response_add(request, obj, *args, **kwargs)
+        itemeditor_post_save_related.send(sender=obj.__class__, instance=obj, created=True)
+        return r
+
+    def response_change(self, request, obj, *args, **kwargs):
+        r = super(ItemEditor, self).response_change(request, obj, *args, **kwargs)
+        itemeditor_post_save_related.send(sender=obj.__class__, instance=obj, created=False)
+        return r
+
     @property
     def change_form_template(self):
         return self.get_template_list()
@@ -205,9 +260,8 @@ class ItemEditor(admin.ModelAdmin):
             ]
 
     def get_fieldsets(self, request, obj=None):
-        """ Convert show_on_top to fieldset for backwards compatibility.
-
-        Also insert FEINCMS_CONTENT_FIELDSET it not present.
+        """
+        Insert FEINCMS_CONTENT_FIELDSET it not present.
         Is it reasonable to assume this should always be included?
         """
 
@@ -217,31 +271,14 @@ class ItemEditor(admin.ModelAdmin):
         if not FEINCMS_CONTENT_FIELDSET_NAME in dict(fieldsets).keys():
             fieldsets.append(FEINCMS_CONTENT_FIELDSET)
 
-        if getattr(self, 'show_on_top', ()):
-            import warnings
-            warnings.warn("The show_on_top will soon be removed; please "
-                          "update your " "code to use fieldsets instead. ",
-                          PendingDeprecationWarning)
-            if hasattr(self.model, '_feincms_templates'):
-                if 'template_key' not in self.show_on_top:
-                    self.show_on_top = ['template_key'] + \
-                        list(self.show_on_top)
-            if self.declared_fieldsets:
-                # check to ensure no duplicated fields
-                all_fields = []
-                for fieldset_data in dict(self.declared_fieldsets).values():
-                    all_fields += list(fieldset_data.get('fields', ()))
-                for field_name in self.show_on_top:
-                    if field_name in all_fields:
-                        raise ImproperlyConfigured(
-                            'Field "%s" is present in both show_on_top and '
-                            'fieldsets' % field_name)
-            else: # no _declared_ fieldsets,
-                # remove show_on_top fields from implicit fieldset
-                for fieldset in fieldsets:
-                    for field_name in self.show_on_top:
-                        if field_name in fieldset[1]['fields']:
-                            fieldset[1]['fields'].remove(field_name)
-            fieldsets.insert(0, (None, {'fields': self.show_on_top}))
-
         return fieldsets
+
+    # These next are only used if later we use a subclass of this class
+    # which also inherits from VersionAdmin.
+    revision_form_template = "admin/feincms/revision_form.html"
+
+    recover_form_template = "admin/feincms/recover_form.html"
+
+    def render_revision_form(self, request, obj, version, context, revert=False, recover=False):
+        context.update(self.get_extra_context(request))
+        return super(ItemEditor, self).render_revision_form(request, obj, version, context, revert, recover)
